@@ -89,6 +89,8 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
   private SortedMap<String, Integer> lastBandwidthWeights = null;
 
+  private Set<String> lastRecommendedServerVersions = null;
+
   private int relayConsensusesProcessed = 0;
 
   private int bridgeStatusesProcessed = 0;
@@ -199,6 +201,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     }
     detailsStatus.setHibernating(descriptor.isHibernating() ? true :
         null);
+    detailsStatus.setAdvertisedOrAddresses(descriptor.getOrAddresses());
     this.documentStore.store(detailsStatus, fingerprint);
   }
 
@@ -247,14 +250,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     if (validAfterMillis > this.relaysLastValidAfterMillis) {
       this.relaysLastValidAfterMillis = validAfterMillis;
     }
-    Set<String> recommendedVersions = null;
-    if (consensus.getRecommendedServerVersions() != null) {
-      recommendedVersions = new HashSet<>();
-      for (String recommendedVersion :
-          consensus.getRecommendedServerVersions()) {
-        recommendedVersions.add("Tor " + recommendedVersion);
-      }
-    }
     for (Map.Entry<String, NetworkStatusEntry> e :
         consensus.getStatusEntries().entrySet()) {
       String fingerprint = e.getKey();
@@ -287,10 +282,12 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         nodeStatus.setConsensusWeight(entry.getBandwidth());
         nodeStatus.setDefaultPolicy(entry.getDefaultPolicy());
         nodeStatus.setPortList(entry.getPortList());
-        nodeStatus.setRecommendedVersion((recommendedVersions == null
-            || entry.getVersion() == null) ? null :
-            recommendedVersions.contains(entry.getVersion()));
-        nodeStatus.setVersion(entry.getVersion());
+        String version = null;
+        if (null != entry.getVersion()
+            && entry.getVersion().startsWith("Tor ")) {
+          version = entry.getVersion().split(" ")[1];
+        }
+        nodeStatus.setVersion(version);
       }
       if (entry.getUnmeasured()) {
         if (!this.lastSeenUnmeasured.containsKey(fingerprint)
@@ -309,6 +306,8 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     this.relayConsensusesProcessed++;
     if (this.relaysLastValidAfterMillis == validAfterMillis) {
       this.lastBandwidthWeights = consensus.getBandwidthWeights();
+      this.lastRecommendedServerVersions
+          = new HashSet<>(consensus.getRecommendedServerVersions());
     }
   }
 
@@ -783,6 +782,26 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
       nodeStatus.setContact(detailsStatus.getContact());
 
+      /* Extract tor software version for bridges from their "platform" line.
+       * (We already know this for relays from "v" lines in the consensus.) */
+      if (!nodeStatus.isRelay()) {
+        String version = null;
+        if (null != detailsStatus.getPlatform()
+            && detailsStatus.getPlatform().startsWith("Tor ")) {
+          version = detailsStatus.getPlatform().split(" ")[1];
+        }
+        nodeStatus.setVersion(version);
+      }
+
+      /* Compare tor software version (for relays and bridges) with the
+       * recommended-server-versions line in the last known consensus and set
+       * the recommended_version field accordingly. */
+      if (null != this.lastRecommendedServerVersions
+          && null != nodeStatus.getVersion()) {
+        nodeStatus.setRecommendedVersion(this.lastRecommendedServerVersions
+            .contains(nodeStatus.getVersion()));
+      }
+
       Map<String, Long> exitAddresses = new HashMap<>();
       if (detailsStatus.getExitAddresses() != null) {
         for (Map.Entry<String, Long> e :
@@ -849,6 +868,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       if (this.rdnsLookupResults.containsKey(fingerprint)) {
         String hostName = this.rdnsLookupResults.get(fingerprint);
         detailsStatus.setHostName(hostName);
+        nodeStatus.setHostName(hostName);
         nodeStatus.setLastRdnsLookup(this.startedRdnsLookups);
       }
 
@@ -888,6 +908,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           nodeStatus.getRecommendedVersion());
       detailsStatus.setLastChangedOrAddressOrPort(
           nodeStatus.getLastChangedOrAddressOrPort());
+      detailsStatus.setVersion(nodeStatus.getVersion());
 
       this.documentStore.store(detailsStatus, fingerprint);
       this.documentStore.store(nodeStatus, fingerprint);
