@@ -129,6 +129,7 @@ public class ResourceServletTest {
   @SuppressWarnings("JavadocMethod")
   @Before
   public void createSampleRelaysAndBridges() {
+    this.relays = new TreeMap<>();
     org.torproject.onionoo.docs.SummaryDocument relayTorkaZ =
         new org.torproject.onionoo.docs.SummaryDocument(true, "TorkaZ",
         "000C5F55BD4814B917CC474BD537F1A3B33CCE2A", Arrays.asList(
@@ -143,7 +144,9 @@ public class ResourceServletTest {
             "0025C136C1F3A9EEFE2AE3F918F03BFA21B5070B" })),
         new TreeSet<>(Arrays.asList(
         new String[] { "001C13B3A55A71B977CA65EC85539D79C653A3FC" })),
-        "0.2.3.25");
+        "0.2.3.25", "ppp-62-216-201-221.dynamic.mnet-online.de", true);
+    this.relays.put("000C5F55BD4814B917CC474BD537F1A3B33CCE2A",
+        relayTorkaZ);
     org.torproject.onionoo.docs.SummaryDocument relayFerrari458 =
         new org.torproject.onionoo.docs.SummaryDocument(true, "Ferrari458",
         "001C13B3A55A71B977CA65EC85539D79C653A3FC", Arrays.asList(
@@ -155,10 +158,8 @@ public class ResourceServletTest {
         new TreeSet<String>(Arrays.asList(new String[] {
             "000C5F55BD4814B917CC474BD537F1A3B33CCE2A" })),
         new TreeSet<>(Arrays.asList(new String[] {
-            "000C5F55BD4814B917CC474BD537F1A3B33CCE2A" })), null);
-    this.relays = new TreeMap<>();
-    this.relays.put("000C5F55BD4814B917CC474BD537F1A3B33CCE2A",
-        relayTorkaZ);
+            "000C5F55BD4814B917CC474BD537F1A3B33CCE2A" })), null,
+            "c-68-38-171-200.hsd1.in.comcast.net", null);
     this.relays.put("001C13B3A55A71B977CA65EC85539D79C653A3FC",
         relayFerrari458);
     org.torproject.onionoo.docs.SummaryDocument relayTimMayTribute =
@@ -169,9 +170,10 @@ public class ResourceServletTest {
         new TreeSet<>(Arrays.asList(new String[] { "Fast",
             "Running", "Unnamed", "V2Dir", "Valid" })), 63L, "a1",
         DateTimeHelper.parse("2013-04-16 18:00:00"), "AS6830",
-        "1024D/51E2A1C7 steven j. murdoch "
+        "1024d/51e2a1c7 \"steven j. murdoch\" "
         + "<tor+steven.murdoch@cl.cam.ac.uk> <fb-token:5sr_k_zs2wm=>",
-        new TreeSet<String>(), new TreeSet<String>(), "0.2.3.25");
+        new TreeSet<String>(), new TreeSet<String>(), "0.2.3.24-rc-dev", null,
+        false);
     this.relays.put("0025C136C1F3A9EEFE2AE3F918F03BFA21B5070B",
         relayTimMayTribute);
     this.bridges = new TreeMap<>();
@@ -182,7 +184,7 @@ public class ResourceServletTest {
         DateTimeHelper.parse("2013-04-21 18:07:03"), false,
         new TreeSet<>(Arrays.asList(new String[] { "Valid" })), -1L,
         null, DateTimeHelper.parse("2013-04-20 15:37:04"), null, null,
-        null, null, null);
+        null, null, "0.2.2.39", null, true);
     this.bridges.put("0000831B236DFF73D409AD17B40E2A728A53994F",
         bridgeec2bridgercc7f31fe);
     org.torproject.onionoo.docs.SummaryDocument bridgeUnnamed =
@@ -192,7 +194,7 @@ public class ResourceServletTest {
         DateTimeHelper.parse("2013-04-20 17:37:04"), false,
         new TreeSet<>(Arrays.asList(new String[] { "Valid" })), -1L,
         null, DateTimeHelper.parse("2013-04-14 07:07:05"), null, null,
-        null, null, null);
+        null, null, null, null, null);
     this.bridges.put("0002D9BDBBC230BD9C78FF502A16E0033EF87E0C",
         bridgeUnnamed);
     org.torproject.onionoo.docs.SummaryDocument bridgegummy =
@@ -203,7 +205,7 @@ public class ResourceServletTest {
         new TreeSet<>(Arrays.asList(new String[] { "Running",
             "Valid" })), -1L, null,
         DateTimeHelper.parse("2013-01-16 21:07:04"), null, null, null,
-        null, null);
+        null, "0.2.4.4-alpha-dev", null, false);
     this.bridges.put("1FEDE50ED8DBA1DD9F9165F78C8131E4A44AB756",
         bridgegummy);
   }
@@ -932,6 +934,40 @@ public class ResourceServletTest {
         new String[] { "TimMayTribute" }, 0, null);
   }
 
+  @Test(timeout = 100)
+  public void testSearchDoubleQuotedEmailAddress() {
+    this.assertSummaryDocument(
+        "/summary?search=contact:\"klaus dot zufall at gmx dot de\"", 1,
+        new String[] { "TorkaZ" }, 0, null);
+  }
+
+  @Test(timeout = 100)
+  public void testSearchDoubleQuotedContactAndNickname() {
+    this.assertSummaryDocument(
+        "/summary?search=contact:\"dot de\" TorkaZ", 1,
+        new String[] { "TorkaZ" }, 0, null);
+  }
+
+  @Test(timeout = 100)
+  public void testSearchMissingEndingDoubleQuote() {
+    this.assertErrorStatusCode(
+        "/summary?search=contact:\"klaus dot zufall at gmx dot de", 400);
+  }
+
+  @Test(timeout = 100)
+  public void testSearchEvenNumberOfDoubleQuotes() {
+    this.assertSummaryDocument(
+        "/summary?search=contact:\"\"\" \"\"\"", 0,
+        null, 0, null);
+  }
+
+  @Test(timeout = 100)
+  public void testSearchContactEscapedDoubleQuotes() {
+    this.assertSummaryDocument(
+        "/summary?search=contact:\"1024D/51E2A1C7 \\\"Steven J. Murdoch\\\"\"",
+        1, new String[] { "TimMayTribute" }, 0, null);
+  }
+
   @Test(timeout = 100)
   public void testLookupFingerprint() {
     this.assertSummaryDocument(
@@ -1070,7 +1106,7 @@ public class ResourceServletTest {
         null);
   }
 
-  @Test(timeout = 100)
+  @Test(timeout = 200)
   public void testCountryDeDe() {
     this.assertSummaryDocument(
         "/summary?country=de&country=de", 1, new String[] { "TorkaZ" }, 0,
@@ -1536,14 +1572,19 @@ public class ResourceServletTest {
 
   @Test
   public void testVersion02325() {
-    this.assertSummaryDocument("/summary?version=0.2.3.25", 2,
-        new String[] { "TorkaZ", "TimMayTribute" }, 0, null);
+    this.assertSummaryDocument("/summary?version=0.2.3.25", 1,
+        new String[] { "TorkaZ" }, 0, null);
   }
 
   @Test
   public void testVersion02324() {
-    this.assertSummaryDocument("/summary?version=0.2.3.24-rc-dev", 0,
-        new String[] { "Ferrari458" }, 0, null);
+    this.assertSummaryDocument("/summary?version=0.2.3.24-rc-dev", 1,
+        new String[] { "TimMayTribute" }, 0, null);
+  }
+
+  @Test
+  public void testVersion02326() {
+    this.assertSummaryDocument("/summary?version=0.2.3.26", 0, null, 0, null);
   }
 
   @Test
@@ -1559,12 +1600,12 @@ public class ResourceServletTest {
 
   @Test
   public void testVersion0() {
-    this.assertSummaryDocument("/summary?version=0", 2, null, 0, null);
+    this.assertSummaryDocument("/summary?version=0", 2, null, 2, null);
   }
 
   @Test
   public void testVersion02() {
-    this.assertSummaryDocument("/summary?version=0.2", 2, null, 0, null);
+    this.assertSummaryDocument("/summary?version=0.2", 2, null, 2, null);
   }
 
   @Test
@@ -1589,5 +1630,94 @@ public class ResourceServletTest {
     /* This is also correct when comparing strings. */
     this.assertErrorStatusCode("/summary?version=*", 400);
   }
+
+  @Test
+  public void testHostNameDe() {
+    this.assertSummaryDocument("/summary?host_name=de", 1, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameE() {
+    this.assertSummaryDocument("/summary?host_name=e", 1, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameDotDe() {
+    this.assertSummaryDocument("/summary?host_name=.de", 1, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameOnlineDe() {
+    this.assertSummaryDocument("/summary?host_name=online.de", 1, null, 0,
+        null);
+  }
+
+  @Test
+  public void testHostNameOnlineDeSomeCapitalized() {
+    this.assertSummaryDocument("/summary?host_name=onLiNe.dE", 1, null, 0,
+        null);
+  }
+
+  @Test
+  public void testHostNameOnline() {
+    this.assertSummaryDocument("/summary?host_name=online", 0, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameTorkaZFull() {
+    this.assertSummaryDocument(
+        "/summary?host_name=ppp-62-216-201-221.dynamic.mnet-online.de",
+        1, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameTorkaZSub() {
+    this.assertSummaryDocument(
+        "/summary?host_name=sub.ppp-62-216-201-221.dynamic.mnet-online.de",
+        0, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameNet() {
+    this.assertSummaryDocument("/summary?host_name=net", 1, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameCom() {
+    this.assertSummaryDocument("/summary?host_name=com", 0, null, 0, null);
+  }
+
+  @Test
+  public void testHostNameDollar() {
+    this.assertErrorStatusCode("/summary?host_name=$", 400);
+  }
+
+  @Test
+  public void testHostNameUmlaut() {
+    this.assertErrorStatusCode("/summary?host_name=äöü", 400);
+  }
+
+  @Test
+  public void testRecommendedVersionTrue() {
+    this.assertSummaryDocument("/summary?recommended_version=true", 1,
+        new String[] { "TorkaZ" }, 1, new String[] { "ec2bridgercc7f31fe" });
+  }
+
+  @Test
+  public void testRecommendedVersionFalse() {
+    this.assertSummaryDocument("/summary?recommended_version=false", 1,
+        new String[] { "TimMayTribute" }, 1, new String[] { "gummy" });
+  }
+
+  @Test
+  public void testRecommendedVersionTrueCapitalized() {
+    this.assertSummaryDocument("/summary?recommended_version=TRUE", 1,
+        new String[] { "TorkaZ" }, 1,new String[] { "ec2bridgercc7f31fe" });
+  }
+
+  @Test
+  public void testRecommendedVersionNull() {
+    this.assertErrorStatusCode("/summary?recommended_version=null", 400);
+  }
 }
 
